@@ -20,6 +20,23 @@
       along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+/*
+   8 July 2017 [Paul Hardy]:
+      - Reads new second field that contains an x-axis offset for
+        each combining character in "*combining.txt" files.
+      - Uses the above x-axis offset value for a combining character
+        to print combining circle in the left half of a double
+        diacritic combining character grid, or in the center for
+        other combining characters.
+      - Adds exceptions for U+01107F (Brahmi number joiner) and
+        U+01D1A0 (vertical stroke musical ornament); they are in
+        a combining.txt file for positioning, but are not actually
+        Unicode combining characters.
+      - Typo fix: "single-width"-->"double-width" in comment for
+        add_double_circle function.
+*/
+
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,12 +51,14 @@ main (int argc, char **argv)
 
    char teststring[MAXSTRING];  /* current input line                       */
    int  loc;                    /* Unicode code point of current input line */
+   int  offset;                 /* offset value of a combining character    */
    char *gstart;                /* glyph start, pointing into teststring    */
 
-   char combining[0x110000];
+   char combining[0x110000];    /* 1 --> combining glyph; 0 --> non-combining */
+   char x_offset [0x110000];    /* second value in *combining.txt files       */
 
-   void add_single_circle(char *); /* add a single-width dashed circle */
-   void add_double_circle(char *); /* add a double-width dashed circle */
+   void add_single_circle(char *);      /* add a single-width dashed circle */
+   void add_double_circle(char *, int); /* add a double-width dashed circle */
 
    FILE *infilefp;
 
@@ -56,6 +75,7 @@ main (int argc, char **argv)
    */
    /* Start with no combining code points flagged */
    memset (combining, 0, 0x110000 * sizeof (char));
+   memset (x_offset , 0, 0x110000 * sizeof (char));
 
    if ((infilefp = fopen (argv[1],"r")) == NULL) {
       fprintf (stderr,"ERROR - combining characters file %s not found.\n\n",
@@ -64,7 +84,18 @@ main (int argc, char **argv)
    }
 
    /* Flag list of combining characters to add a dashed circle. */
-   while (fscanf (infilefp, "%X", &loc) != EOF) combining[loc] = 1;
+   while (fscanf (infilefp, "%X:%d", &loc, &offset) != EOF) {
+      /*
+         U+01107F and U+01D1A0 are not defined as combining characters
+         in Unicode; they were added in a combining.txt file as the
+         only way to make them look acceptable in proximity to other
+         glyphs in their script.
+      */
+      if (loc != 0x01107F && loc != 0x01D1A0) {
+         combining[loc] = 1;
+         x_offset [loc] = offset;
+      }
+   }
 
    fclose (infilefp); /* all done reading combining.txt */
 
@@ -88,8 +119,10 @@ main (int argc, char **argv)
       sscanf (teststring, "%X", &loc);     /* loc == the Uniocde code point    */
       gstart = index (teststring,':') + 1; /* start of glyph bitmap            */
       if (combining[loc]) {                /* if a combining character         */
-         if (strlen (gstart) < 35) add_single_circle (gstart); /* single-width */
-         else add_double_circle (gstart);                      /* double-width */
+         if (strlen (gstart) < 35)
+            add_single_circle (gstart);                 /* single-width */
+         else
+            add_double_circle (gstart, x_offset[loc]);  /* double-width */
       }
       printf ("%s", teststring); /* output the new character .hex string */
    }
@@ -155,35 +188,66 @@ add_single_circle (char *glyphstring)
 
 
 /*
-   add_double_circle - superimpose a single-width dashed combining circle.
+   add_double_circle - superimpose a double-width dashed combining circle.
 */
 void
-add_double_circle (char *glyphstring)
+add_double_circle (char *glyphstring, int offset)
 {
 
    char newstring[256];
    /* Circle hex string pattern is "00000008000024004200240000000000" */
-   char circle[64]={0x0,0x0,0x0,0x0,  /* row  1 */
-                    0x0,0x0,0x0,0x0,  /* row  2 */
-                    0x0,0x0,0x0,0x0,  /* row  3 */
-                    0x0,0x0,0x0,0x0,  /* row  4 */
-                    0x0,0x0,0x0,0x0,  /* row  5 */
-                    0x0,0x0,0x0,0x0,  /* row  6 */
-                    0x0,0x2,0x4,0x0,  /* row  7 */
-                    0x0,0x0,0x0,0x0,  /* row  8 */
-                    0x0,0x4,0x2,0x0,  /* row  9 */
-                    0x0,0x0,0x0,0x0,  /* row 10 */
-                    0x0,0x2,0x4,0x0,  /* row 11 */
-                    0x0,0x0,0x0,0x0,  /* row 12 */
-                    0x0,0x0,0x0,0x0,  /* row 13 */
-                    0x0,0x0,0x0,0x0,  /* row 14 */
-                    0x0,0x0,0x0,0x0,  /* row 15 */
-                    0x0,0x0,0x0,0x0}; /* row 16 */
+
+   /* For double diacritical glyphs (offset = -8) */
+   /* Combining circle is left-justified.         */
+   char circle08[64]={0x0,0x0,0x0,0x0,  /* row  1 */
+                      0x0,0x0,0x0,0x0,  /* row  2 */
+                      0x0,0x0,0x0,0x0,  /* row  3 */
+                      0x0,0x0,0x0,0x0,  /* row  4 */
+                      0x0,0x0,0x0,0x0,  /* row  5 */
+                      0x0,0x0,0x0,0x0,  /* row  6 */
+                      0x2,0x4,0x0,0x0,  /* row  7 */
+                      0x0,0x0,0x0,0x0,  /* row  8 */
+                      0x4,0x2,0x0,0x0,  /* row  9 */
+                      0x0,0x0,0x0,0x0,  /* row 10 */
+                      0x2,0x4,0x0,0x0,  /* row 11 */
+                      0x0,0x0,0x0,0x0,  /* row 12 */
+                      0x0,0x0,0x0,0x0,  /* row 13 */
+                      0x0,0x0,0x0,0x0,  /* row 14 */
+                      0x0,0x0,0x0,0x0,  /* row 15 */
+                      0x0,0x0,0x0,0x0}; /* row 16 */
+
+   /* For all other combining glyphs (offset = -16) */
+   /* Combining circle is centered in 16 columns.   */
+   char circle16[64]={0x0,0x0,0x0,0x0,  /* row  1 */
+                      0x0,0x0,0x0,0x0,  /* row  2 */
+                      0x0,0x0,0x0,0x0,  /* row  3 */
+                      0x0,0x0,0x0,0x0,  /* row  4 */
+                      0x0,0x0,0x0,0x0,  /* row  5 */
+                      0x0,0x0,0x0,0x0,  /* row  6 */
+                      0x0,0x2,0x4,0x0,  /* row  7 */
+                      0x0,0x0,0x0,0x0,  /* row  8 */
+                      0x0,0x4,0x2,0x0,  /* row  9 */
+                      0x0,0x0,0x0,0x0,  /* row 10 */
+                      0x0,0x2,0x4,0x0,  /* row 11 */
+                      0x0,0x0,0x0,0x0,  /* row 12 */
+                      0x0,0x0,0x0,0x0,  /* row 13 */
+                      0x0,0x0,0x0,0x0,  /* row 14 */
+                      0x0,0x0,0x0,0x0,  /* row 15 */
+                      0x0,0x0,0x0,0x0}; /* row 16 */
+
+   char *circle; /* points into circle16 or circle08 */
 
    int digit1, digit2; /* corresponding digits in each string */
 
    int i; /* index variables */
 
+
+   /*
+      Determine if combining circle is left-justified (offset = -8)
+      or centered (offset = -16).
+   */
+   circle = (offset >= -8) ? circle08 : circle16;
+
    /* for each character position, OR the corresponding circle glyph value */
    for (i = 0; i < 64; i++) {
       glyphstring[i] = toupper (glyphstring[i]);
